/**
 * @author jonasli
 * @module webpack config
 * @date 2016/03/01
 */

'use strict';

let path = require('path')
let fs = require('fs')

let webpack = require('webpack')
let _ = require('lodash')
let glob = require('glob')

let ExtractTextPlugin = require('extract-text-webpack-plugin')
let HtmlWebpackPlugin = require('html-webpack-plugin')

let UglifyJsPlugin = webpack.optimize.UglifyJsPlugin
let CommonsChunkPlugin = webpack.optimize.CommonsChunkPlugin

let srcDir = path.resolve(process.cwd(), 'src')
let nodeModPath = path.resolve(__dirname, './node_modules')
let dist = 'dist/'
let pathMap = require('./src/pathmap.json')

let entries = (() => {
    let jsDir = path.resolve(srcDir, 'components')
    let entryFiles = glob.sync(jsDir + '/**/index.js')
    let map = {}

    entryFiles.forEach((filePath) => {
        // let filename = filePath.substring(filePath.lastIndexOf('\/') + 1, filePath.lastIndexOf('.'))
        let filename = filePath.substring(filePath.lastIndexOf('components/') + 11, filePath.lastIndexOf('/index'))

        map[filename] = filePath
    })

    return map;
})()

let chunks = Object.keys(entries)

module.exports = (options) => {
    options = options || {}

    let debug = options.debug !== undefined ? options.debug : true
    let publicPath = '' // debug ? '' : 'http://cdn.site.com/'
    let cssLoader
    let scssLoader

    // generate entry html files
    // 自动生成入口文件，入口js名必须和入口文件名相同
    // 例如，a页的入口文件是a.html，那么在js目录下必须有一个a.js作为入口文件
    let plugins = (() => {
        let entryHtml = glob.sync(srcDir + '/*.html')
        let r = []

        entryHtml.forEach((filePath) => {
            let filename = filePath.substring(filePath.lastIndexOf('\/') + 1, filePath.lastIndexOf('.'))
            let conf = {
                template: filePath,
                filename: filename + '.html'
            }

            if(filename in entries) {
                conf.inject = 'body'
                conf.chunks = [filename] // 'vender', 'common',
                // conf.chunksSortMode = 'none'
            }

            // if(/index/.test(filename)) conf.chunks.splice(0, 1)

            r.push(new HtmlWebpackPlugin(conf))
        })

        return r
    })()

    // if(debug) {
        // 开发阶段，css直接内嵌
        cssLoader = 'style!css'
        scssLoader = 'style!css!sass'
    // } else {
    //     // 编译阶段，css分离出来单独引入
    //     cssLoader = ExtractTextPlugin.extract('style', 'css?minimize') // enable minimize
    //     scssLoader = ExtractTextPlugin.extract('style', 'css?minimize', 'sass')

    //     plugins.push(
    //         new ExtractTextPlugin('css/[contenthash:8].[name].min.css', {
    //             // 当allChunks指定为false时，css loader必须指定怎么处理
    //             // additional chunk所依赖的css，即指定`ExtractTextPlugin.extract()`
    //             // 第一个参数`notExtractLoader`，一般是使用style-loader
    //             // @see https://github.com/webpack/extract-text-webpack-plugin
    //             allChunks: false
    //         })
    //     )

        plugins.push(new UglifyJsPlugin())
    // }

    let config = {
        entry: Object.assign(entries, {
            // 'vender': ['jquery', 'underscore', 'backbone', 'bootstrap'] //  , 'bstable', 'bstableCn', 'layer', 'jqueryui'
        }),

        output: {
            path: path.resolve(dist),
            // filename: debug ? '[name].js' : 'js/[chunkhash:8].[name].min.js',
            filename: 'components/[name]/index.js',
            // chunkFilename: debug ? '[chunkhash:8].chunk.js' : 'js/[chunkhash:8].chunk.min.js',
            // hotUpdateChunkFilename: debug ? '[id].js' : 'js/[id].[chunkhash:8].min.js',
            publicPath: publicPath
        },

        resolve: {
            root: [srcDir, './node_modules'],
            alias: pathMap,
            extensions: ['', '.js', '.css', '.scss', '.tpl', '.vue', '.html', '.png', '.jpg']
        },

        resolveLoader: {
            root: path.join(__dirname, 'node_modules')
        },

        module: {
            loaders: [
                {
                    test: /\.vue$/,
                    loader: 'vue',
                    exclude: /node_modules/
                  },
                {
                    test: /\.(jpe?g|png|gif|svg)$/i,
                    loaders: [
                        'image?{bypassOnDebug: true, progressive:true, \
                            optimizationLevel: 3, pngquant:{quality: "65-80", speed: 4}}',
                        // url-loader更好用，小于10KB的图片会自动转成dataUrl，
                        // 否则则调用file-loader，参数直接传入
                        // 'url?limit=10000&name=img/[hash:8].[name].[ext]',
                        'url?limit=1&name=img/[hash:8].[name].[ext]',
                    ]
                },
                {
                    test: /\.(woff|woff2|eot|ttf)$/i,
                    loader: 'url?limit=10000&name=fonts/[hash:8].[name].[ext]'
                },
                {test: /\.(tpl|ejs|html)$/, loader: 'ejs'},
                {test: /\.css$/, loader: cssLoader},
                {test: /\.scss$/, loader: scssLoader}
            ]
            , noParse: [/third/]
        },

        plugins: [
            // new CommonsChunkPlugin({
            //     name: 'common',
            //     chunks: ['manage', 'office', 'stat']
            // }),
            // new CommonsChunkPlugin({
            //     name: 'vender',
            //     chunks: ['common']
            // })
        ].concat(plugins),

        devServer: {
            hot: true,
            noInfo: false,
            inline: true,
            publicPath: publicPath,
            stats: {
                cached: false,
                colors: true
            }
        }
    }


    // fs.open('./test.txt', 'a', function opened(err, fd) {
    //     if (err) { throw err; }
    //     var writeBuffer = new Buffer(JSON.stringify(config)),
    //     bufferPosition = 0,
    //     bufferLength = writeBuffer.length, filePosition = null;
    //     fs.write( fd,
    //         writeBuffer,
    //         bufferPosition,
    //         bufferLength,
    //         filePosition,
    //         function(err, written) {
    //             if (err) { throw err; }
    //             console.log('wrote ' + written + ' bytes');
    //         }
    //     );
    // });

    return config
}
